@@ -1,5 +1,5 @@
 /*
- * Copyright 1999-2006 Sun Microsystems, Inc.  All Rights Reserved.
+ * Copyright 1999-2008 Sun Microsystems, Inc.  All Rights Reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -100,6 +100,7 @@ public class Enter extends JCTree.Visitor {
     ClassReader reader;
     Annotate annotate;
     MemberEnter memberEnter;
+    Types types;
     Lint lint;
     JavaFileManager fileManager;
     private final CancelService cancelService;
@@ -125,6 +126,7 @@ public class Enter extends JCTree.Visitor {
         syms = Symtab.instance(context);
         chk = Check.instance(context);
         memberEnter = MemberEnter.instance(context);
+        types = Types.instance(context);
         annotate = Annotate.instance(context);
         lint = Lint.instance(context);
         cancelService = CancelService.instance(context);
@@ -292,7 +294,9 @@ public class Enter extends JCTree.Visitor {
     <T extends JCTree> List<Type> classEnter(List<T> trees, Env<AttrContext> env) {
         ListBuffer<Type> ts = new ListBuffer<Type>();
         for (List<T> l = trees; l.nonEmpty(); l = l.tail) {
-            ts.append(classEnter(l.head, env));
+            Type t = classEnter(l.head, env);
+            if (t != null)
+                ts.append(t);
             memoryWatch.abortIfMemoryLow();
         }
         return ts.toList();
@@ -412,13 +416,14 @@ public class Enter extends JCTree.Visitor {
                 doEnterClass = false;
             }
             if (c == null) {
-                if (tree.name.len != 0 &&
+                if (!tree.name.isEmpty() &&
                         !chk.checkUniqueClassName(tree.pos(), tree.name, enclScope)) {
-                    result = new ErrorType(tree.name, (TypeSymbol)owner);
+                    result = types.createErrorType(tree.name, owner, Type.noType);
                     tree.sym = (ClassSymbol)result.tsym;
                     Env<AttrContext> localEnv = classEnv(tree, env);
                     typeEnvs.put(tree.sym, localEnv);
                     tree.sym.completer = memberEnter;
+                    ((ClassType)result).typarams_field = classEnter(tree.typarams, localEnv);
                     if (!tree.sym.isLocal() && uncompleted != null) uncompleted.append(tree.sym);
                     return;
                 }
@@ -453,7 +458,7 @@ public class Enter extends JCTree.Visitor {
                                 reattr = true;
                         }
                     }
-                    if (c.name.len != 0)
+                    if (!c.name.isEmpty())
                         chk.checkTransparentClass(tree.pos(), c, env.info.scope);
                 }
             }
@@ -472,8 +477,13 @@ public class Enter extends JCTree.Visitor {
         if (!reattr && !noctx) {
             if (chk.compiled.get(c.flatname) != null) {
                 duplicateClass(tree.pos(), c);
-                result = new ErrorType(tree.name, (TypeSymbol)owner);
+                result = types.createErrorType(tree.name, owner, Type.noType);
                 tree.sym  = (ClassSymbol)result.tsym;
+                Env<AttrContext> localEnv = classEnv(tree, env);
+                typeEnvs.put(tree.sym, localEnv);
+                tree.sym.completer = memberEnter;
+                ((ClassType)result).typarams_field = classEnter(tree.typarams, localEnv);
+                if (!tree.sym.isLocal() && uncompleted != null) uncompleted.append(tree.sym);
                 return;
             }
             chk.compiled.put(c.flatname, c);
@@ -586,20 +596,19 @@ public class Enter extends JCTree.Visitor {
                     break;
                 }
             }
-            if (result == null) {
-                ClassSymbol cs = env.info.scope.owner.outermostClass();
-                treeLoader.couplingError(cs, tree);
-            }
-        } else {
-            TypeVar a = (tree.type != null)
-            ? (TypeVar)tree.type
-                    : new TypeVar(tree.name, env.info.scope.owner, syms.botType);
-            tree.type = a;
-            if (chk.checkUnique(tree.pos(), a.tsym, env.info.scope)) {
-                env.info.scope.enter(a.tsym);
-            }
-            result = a;
+            if (result != null)
+                return;
+            ClassSymbol cs = env.info.scope.owner.outermostClass();
+            treeLoader.couplingError(cs, tree);
         }
+        TypeVar a = (tree.type != null)
+        ? (TypeVar)tree.type
+                : new TypeVar(tree.name, env.info.scope.owner, syms.botType);
+        tree.type = a;
+        if (chk.checkUnique(tree.pos(), a.tsym, env.info.scope)) {
+            env.info.scope.enter(a.tsym);
+        }
+        result = a;
     }
 
     /** Default class enter visitor method: do nothing.
