@@ -1,5 +1,5 @@
 /*
- * Copyright 1999-2006 Sun Microsystems, Inc.  All Rights Reserved.
+ * Copyright 1999-2008 Sun Microsystems, Inc.  All Rights Reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -35,6 +35,7 @@ import com.sun.tools.javac.util.JCDiagnostic.DiagnosticPosition;
 import com.sun.tools.javac.code.Symbol.*;
 import com.sun.tools.javac.tree.JCTree.*;
 
+import java.util.logging.Logger;
 import static com.sun.tools.javac.code.Flags.*;
 import static com.sun.tools.javac.code.Kinds.*;
 import static com.sun.tools.javac.code.TypeTags.*;
@@ -177,7 +178,7 @@ public class Flow extends TreeScanner {
     protected static final Context.Key<Flow> flowKey =
         new Context.Key<Flow>();
 
-    private final Name.Table names;
+    private final Names names;
     private final Log log;
     private final Symtab syms;
     private final Types types;
@@ -195,7 +196,7 @@ public class Flow extends TreeScanner {
     protected Flow(Context context) {
         context.put(flowKey, this);
 
-        names = Name.Table.instance(context);
+        names = Names.instance(context);
         log = Log.instance(context);
         syms = Symtab.instance(context);
         types = Types.instance(context);
@@ -242,6 +243,8 @@ public class Flow extends TreeScanner {
      */
     JCClassDecl classDef;
 
+    JCClassDecl reanalyzedClass;
+
     /** The first variable sequence number in this class definition.
      */
     int firstadr;
@@ -326,10 +329,11 @@ public class Flow extends TreeScanner {
      *  I.e. is symbol either a local or a blank final variable?
      */
     boolean trackable(VarSymbol sym) {
-        return sym != null &&
+        return sym != null && sym.owner != null &&
             (sym.owner.kind == MTH ||
              ((sym.flags() & (FINAL | HASINIT | PARAMETER)) == FINAL &&
-              classDef.sym.isEnclosedBy((ClassSymbol)sym.owner)));
+              classDef.sym.isEnclosedBy((ClassSymbol)sym.owner)) &&
+              (reanalyzedClass == null || ((ClassSymbol)sym.owner).isEnclosedBy(reanalyzedClass.sym)));
     }
 
     /** Initialize new trackable variable by setting its address field
@@ -386,7 +390,7 @@ public class Flow extends TreeScanner {
         tree = TreeInfo.skipParens(tree);
         if (tree.getTag() == JCTree.IDENT || tree.getTag() == JCTree.SELECT) {
             Symbol sym = TreeInfo.symbol(tree);
-            if (!sym.type.isErroneous())
+            if (sym != null && sym.type != null && !sym.type.isErroneous())
                 letInit(tree.pos(), (VarSymbol)sym);
         }
     }
@@ -394,7 +398,7 @@ public class Flow extends TreeScanner {
     /** Check that trackable variable is initialized.
      */
     void checkInit(DiagnosticPosition pos, VarSymbol sym) {
-        if ((sym.adr >= firstadr || sym.owner.kind != TYP) &&
+        if ((sym.adr >= firstadr || (sym.owner != null && sym.owner.kind != TYP)) &&
             trackable(sym) &&
             !inits.isMember(sym.adr)) {
             log.error(pos, "var.might.not.have.been.initialized",
@@ -529,7 +533,7 @@ public class Flow extends TreeScanner {
      *  rather than (un)inits on exit.
      */
     void scanCond(JCTree tree) {
-        if (tree.type.isFalse()) {
+        if (tree.type != null && tree.type.isFalse()) {
             if (inits == null) merge();
             initsWhenTrue = inits.dup();
             initsWhenTrue.inclRange(firstadr, nextadr);
@@ -537,7 +541,7 @@ public class Flow extends TreeScanner {
             uninitsWhenTrue.inclRange(firstadr, nextadr);
             initsWhenFalse = inits;
             uninitsWhenFalse = uninits;
-        } else if (tree.type.isTrue()) {
+        } else if (tree.type != null && tree.type.isTrue()) {
             if (inits == null) merge();
             initsWhenFalse = inits.dup();
             initsWhenFalse.inclRange(firstadr, nextadr);
@@ -571,7 +575,8 @@ public class Flow extends TreeScanner {
         }
         classDef = tree;
         thrown = List.nil();
-        lint = lint.augment(tree.sym.attributes_field);
+        if (tree.sym != null)
+            lint = lint.augment(tree.sym.attributes_field);
 
         try {
             // define all the static fields
@@ -637,7 +642,7 @@ public class Flow extends TreeScanner {
             // in an anonymous class, add the set of thrown exceptions to
             // the throws clause of the synthetic constructor and propagate
             // outwards.
-            if (tree.name == names.empty) {
+            if (tree.name == names.empty && thrownPrev != null) {
                 for (List<JCTree> l = tree.defs; l.nonEmpty(); l = l.tail) {
                     if (TreeInfo.isInitialConstructor(l.head)) {
                         JCMethodDecl mdef = (JCMethodDecl)l.head;
@@ -670,6 +675,10 @@ public class Flow extends TreeScanner {
 
     public void visitMethodDef(JCMethodDecl tree) {
         if (tree.body == null) return;
+        if (tree.sym == null) {
+            Logger.getLogger(Flow.class.getName()).warning("Flow.visitMethodDef has a null tree.sym. Tree: [" + tree + "] Enclosing class: [" + classDef + "]"); //NOI18N
+            return;
+        }
         List<Type> caughtPrev = caught;
         List<Type> mthrown = tree.sym.type.getThrownTypes();
         Bits initsPrev = inits.dup();
@@ -739,6 +748,10 @@ public class Flow extends TreeScanner {
     }
 
     public void visitVarDef(JCVariableDecl tree) {
+        if (tree.sym == null) {
+            Logger.getLogger(Flow.class.getName()).warning("Flow.visitVarDef has a null tree.sym. Tree: [" + tree + "] Enclosing class: [" + classDef + "]"); //NOI18N
+            return;
+        }
         boolean track = trackable(tree.sym);
         if (track && tree.sym.owner.kind == MTH) newVar(tree.sym);
         if (tree.init != null) {
@@ -864,11 +877,18 @@ public class Flow extends TreeScanner {
         ListBuffer<PendingExit> prevPendingExits = pendingExits;
         boolean prevLoopPassTwo = loopPassTwo;
         int nextadrPrev = nextadr;
+        Bits initsExpr = inits;
+        Bits uninitsExpr = uninits;
         scan(tree.expr);
+        if (inits == null) {
+            inits = initsExpr;
+            uninits = uninitsExpr;
+        }
         Bits initsStart = inits.dup();
         Bits uninitsStart = uninits.dup();
 
-        letInit(tree.pos(), tree.var.sym);
+        if (tree.var.sym != null)
+            letInit(tree.pos(), tree.var.sym);
         pendingExits = new ListBuffer<PendingExit>();
         do {
             Bits uninitsEntry = uninits.dup();
@@ -949,8 +969,10 @@ public class Flow extends TreeScanner {
         List<Type> caughtPrev = caught;
         List<Type> thrownPrev = thrown;
         thrown = List.nil();
-        for (List<JCCatch> l = tree.catchers; l.nonEmpty(); l = l.tail)
-            caught = chk.incl(l.head.param.type, caught);
+        for (List<JCCatch> l = tree.catchers; l.nonEmpty(); l = l.tail) {
+            if (l.head.param.type != null)
+                caught = chk.incl(l.head.param.type, caught);
+        }
         Bits uninitsTryPrev = uninitsTry;
         ListBuffer<PendingExit> prevPendingExits = pendingExits;
         pendingExits = new ListBuffer<PendingExit>();
@@ -971,6 +993,8 @@ public class Flow extends TreeScanner {
             alive = true;
             JCVariableDecl param = l.head.param;
             Type exc = param.type;
+            if (exc == null || param.sym == null)
+                continue;
             if (chk.subset(exc, caughtInTry)) {
                 log.error(l.head.pos(),
                           "except.already.caught", exc);
@@ -1045,8 +1069,8 @@ public class Flow extends TreeScanner {
         Bits uninitsBeforeElse = uninitsWhenFalse;
         inits = initsWhenTrue;
         uninits = uninitsWhenTrue;
-        if (tree.truepart.type.tag == BOOLEAN &&
-            tree.falsepart.type.tag == BOOLEAN) {
+        if (tree.truepart.type != null && tree.truepart.type.tag == BOOLEAN &&
+            tree.falsepart.type != null && tree.falsepart.type.tag == BOOLEAN) {
             // if b and c are boolean valued, then
             // v is (un)assigned after a?b:c when true iff
             //    v is (un)assigned after b when true and
@@ -1134,15 +1158,34 @@ public class Flow extends TreeScanner {
     public void visitNewClass(JCNewClass tree) {
         scanExpr(tree.encl);
         scanExprs(tree.args);
-        Symbol ctor = tree.constructor;
        // scan(tree.def);
-        if (ctor != null && ctor.type != null) {
-        for (List<Type> l = ctor.type.getThrownTypes();
-             l.nonEmpty();
-             l = l.tail)
-            markThrown(tree, l.head);
+        if (tree.constructorType != null)
+            for (List<Type> l = tree.constructorType.getThrownTypes();
+                 l.nonEmpty();
+                 l = l.tail) {
+                markThrown(tree, l.head);
+            }
+        List<Type> caughtPrev = caught;
+        try {
+            // If the new class expression defines an anonymous class,
+            // analysis of the anonymous constructor may encounter thrown
+            // types which are unsubstituted type variables.
+            // However, since the constructor's actual thrown types have
+            // already been marked as thrown, it is safe to simply include
+            // each of the constructor's formal thrown types in the set of
+            // 'caught/declared to be thrown' types, for the duration of
+            // the class def analysis.
+            if (tree.def != null && tree.constructorType != null)
+                for (List<Type> l = tree.constructorType.getThrownTypes();
+                     l.nonEmpty();
+                     l = l.tail) {
+                    caught = chk.incl(l.head, caught);
+                }
+            scan(tree.def);
+        }
+        finally {
+            caught = caughtPrev;
         }
-        scan(tree.def);
     }
 
     public void visitNewArray(JCNewArray tree) {
@@ -1233,7 +1276,7 @@ public class Flow extends TreeScanner {
 
     public void visitTypeCast(JCTypeCast tree) {
         super.visitTypeCast(tree);
-        if (!tree.type.isErroneous()
+        if (tree.type != null && !tree.type.isErroneous()
             && lint.isEnabled(Lint.LintCategory.CAST)
             && types.isSameType(tree.expr.type, tree.clazz.type)) {
             log.warning(tree.pos(), "redundant.cast", tree.expr.type);
@@ -1284,10 +1327,13 @@ public class Flow extends TreeScanner {
     }
 
     public void reanalyzeMethod (final TreeMaker make, final JCClassDecl classDef) {
+        JCClassDecl oldReanalyzedClass = reanalyzedClass;
         try {
             init (make);
+            reanalyzedClass = classDef;
             scan(classDef);
         } finally {
+            reanalyzedClass = oldReanalyzedClass;
             cleanup();
         }
     }
