@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2006 Sun Microsystems, Inc.  All Rights Reserved.
+ * Copyright 2003-2008 Sun Microsystems, Inc.  All Rights Reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -63,7 +63,7 @@ public class MemberEnter extends JCTree.Visitor implements Completer {
      */
     final static boolean checkClash = true;
 
-    private final Name.Table names;
+    private final Names names;
     private final Enter enter;
     private final Log log;
     private final Check chk;
@@ -74,8 +74,9 @@ public class MemberEnter extends JCTree.Visitor implements Completer {
     private final Todo todo;
     private final Annotate annotate;
     private final Types types;
+    private final JCDiagnostic.Factory diags;
     private final Target target;
-    private final Messages messages;
+    private final JavacMessages messages;
     private final CancelService cancelService;
     private final LazyTreeLoader treeLoader;
 
@@ -91,7 +92,7 @@ public class MemberEnter extends JCTree.Visitor implements Completer {
 
     protected MemberEnter(Context context) {
         context.put(memberEnterKey, this);
-        names = Name.Table.instance(context);
+        names = Names.instance(context);
         enter = Enter.instance(context);
         log = Log.instance(context);
         chk = Check.instance(context);
@@ -102,6 +103,7 @@ public class MemberEnter extends JCTree.Visitor implements Completer {
         todo = Todo.instance(context);
         annotate = Annotate.instance(context);
         types = Types.instance(context);
+        diags = JCDiagnostic.Factory.instance(context);
         target = Target.instance(context);
         Options options = Options.instance(context);
         skipAnnotations =
@@ -109,7 +111,7 @@ public class MemberEnter extends JCTree.Visitor implements Completer {
         boolean ideMode = options.get("ide") != null;
         boolean backgroundCompilation = options.get("backgroundCompilation") != null;
         ignoreNoLang = ideMode && !backgroundCompilation;
-        messages = Messages.instance(context);
+        messages = JavacMessages.instance(context);
         cancelService = CancelService.instance(context);
         treeLoader = LazyTreeLoader.instance(context);
     }
@@ -129,6 +131,8 @@ public class MemberEnter extends JCTree.Visitor implements Completer {
      *  unnecessarily deep recursion.
      */
     boolean completionEnabled = true;
+    
+    private JCCompilationUnit disabledCompletionInUnit = null;
 
     /* ---------- Processing import clauses ----------------
      */
@@ -146,7 +150,7 @@ public class MemberEnter extends JCTree.Visitor implements Completer {
         if (tsym.kind == PCK && tsym.members().elems == null && !tsym.exists()) {
             // If we can't find java.lang, exit immediately.
             if (((PackageSymbol)tsym).fullname.equals(names.java_lang)) {
-                JCDiagnostic msg = JCDiagnostic.fragment("fatal.err.no.java.lang");
+                JCDiagnostic msg = diags.fragment("fatal.err.no.java.lang");
                 if (ignoreNoLang) {                    
                     throw new CompletionFailure(tsym, msg.toString());
                 }
@@ -339,8 +343,9 @@ public class MemberEnter extends JCTree.Visitor implements Completer {
                     importFrom(tsym);
                     if (!found) {
                         log.error(pos, "cant.resolve.location",
-                                  JCDiagnostic.fragment("kindname.static"),
-                                  name, "", "", Resolve.typeKindName(tsym.type),
+                                  KindName.STATIC,
+                                  name, List.<Type>nil(), List.<Type>nil(),
+                                  Kinds.typeKindName(tsym.type),
                                   tsym.type);
                     }
                 } finally {
@@ -566,30 +571,36 @@ public class MemberEnter extends JCTree.Visitor implements Completer {
         // effects of other imports in Resolve.findGlobalType
         Env<AttrContext> localEnv = env.dup(tree);
 
-        // Attribute qualifying package or class.
-        JCFieldAccess s = (JCFieldAccess) imp;
-        p = attr.
-            attribTree(s.selected,
-                       localEnv,
-                       tree.staticImport ? TYP : (TYP | PCK),
-                       Type.noType).tsym;
-        if (name == names.asterisk) {
-            // Import on demand.
-            chk.checkCanonical(s.selected);
-            if (tree.staticImport)
-                importStaticAll(tree.pos, p, env);
-            else
-                importAll(tree.pos, p, env);
-        } else {
-            // Named type import.
-            if (tree.staticImport) {
-                importNamedStatic(tree.pos(), p, name, localEnv);
+        try {
+            disabledCompletionInUnit = localEnv.toplevel;
+
+            // Attribute qualifying package or class.
+            JCFieldAccess s = (JCFieldAccess) imp;
+            p = attr.
+                attribTree(s.selected,
+                           localEnv,
+                           tree.staticImport ? TYP : (TYP | PCK),
+                           Type.noType).tsym;
+            if (name == names.asterisk) {
+                // Import on demand.
                 chk.checkCanonical(s.selected);
+                if (tree.staticImport)
+                    importStaticAll(tree.pos, p, env);
+                else
+                    importAll(tree.pos, p, env);
             } else {
-                TypeSymbol c = attribImportType(imp, localEnv).tsym;
-                chk.checkCanonical(imp);
-                importNamed(tree.pos(), c, env);
+                // Named type import.
+                if (tree.staticImport) {
+                    importNamedStatic(tree.pos(), p, name, localEnv);
+                    chk.checkCanonical(s.selected);
+                } else {
+                    TypeSymbol c = attribImportType(imp, localEnv).tsym;
+                    chk.checkCanonical(imp);
+                    importNamed(tree.pos(), c, env);
+                }
             }
+        } finally {
+            disabledCompletionInUnit = null;
         }
     }
 
@@ -608,54 +619,60 @@ public class MemberEnter extends JCTree.Visitor implements Completer {
 
         if ((enclScope.owner.flags_field & FROMCLASS) != 0) {
             for (Scope.Entry e = enclScope.lookup(tree.name); e.scope == enclScope; e = e.next()) {
-                if (e.sym.kind == MTH && (types.isSameType(m.type, e.sym.type) || m.name == names.init && (m.owner.name.isEmpty() || (m.owner.owner.kind & (VAR | MTH)) != 0))) {
-                    treeCleaner.scan(tree);
-                    tree.sym = (MethodSymbol)e.sym;
-                    localEnv = methodEnv(tree, env);
-                    tree.sym.flags_field |= FROMCLASS;
-                    if (tree.sym.type.tag == FORALL) {
-                        for(List<Type> tvars = ((ForAll)tree.sym.type).tvars; tvars.nonEmpty(); tvars = tvars.tail)
-                            localEnv.info.scope.enter(tvars.head.tsym);
-                    }
-                    List<VarSymbol> p = tree.sym.params();
-                    if (p != null) {
-                        List<JCVariableDecl> l = tree.params;
-                        while(l.nonEmpty() && p.nonEmpty()) {
-                            p.head.setName(l.head.name);
-                            if (l.head.getModifiers() != null && l.head.getModifiers().getFlags().contains(Modifier.FINAL)) {
-                                //copy the final flag, as the symbol might have come from the classfile:
-                                p.head.flags_field |= FINAL;
+                if (e.sym.kind == MTH) {
+                    boolean sameType = types.isSameType(m.type, e.sym.type);
+                    if (sameType || m.name == names.init && (m.owner.name.isEmpty() || (m.owner.owner.kind & (VAR | MTH)) != 0)) {
+                        treeCleaner.scan(tree);
+                        tree.sym = (MethodSymbol)e.sym;
+                        localEnv = methodEnv(tree, env);
+                        tree.sym.flags_field |= FROMCLASS;
+                        if (tree.sym.type.tag == FORALL) {
+                            for(List<Type> tvars = ((ForAll)tree.sym.type).tvars; tvars.nonEmpty(); tvars = tvars.tail)
+                                localEnv.info.scope.enter(tvars.head.tsym);
+                        }
+                        List<VarSymbol> p = tree.sym.params();
+                        if (p != null) {
+                            List<JCVariableDecl> l = tree.params;
+                            while(l.nonEmpty() && p.nonEmpty()) {
+                                if (sameType) {
+                                    p.head.setName(l.head.name);
+                                    if (l.head.getModifiers() != null && l.head.getModifiers().getFlags().contains(Modifier.FINAL)) {
+                                        //copy the final flag, as the symbol might have come from the classfile:
+                                        p.head.flags_field |= FINAL;
+                                    }
+                                }
+                                localEnv.info.scope.enter(p.head);
+                                p = p.tail;
+                                l = l.tail;
+                            }
+                            while(p.nonEmpty()) {
+                                p.head.setName(p.head.name);
+                                localEnv.info.scope.enter(p.head);
+                                p = p.tail;
                             }
-                            localEnv.info.scope.enter(p.head);
-                            p = p.tail;
-                            l = l.tail;
                         }
-                        while(p.nonEmpty()) {
-                            p.head.setName(p.head.name);
-                            p = p.tail;
+                        tree.sym.type = signature(tree.typarams, tree.params,
+                                tree.restype, tree.thrown,
+                                localEnv);
+                        tree.sym.flags_field &= ~FROMCLASS;
+                        localEnv.info.scope.leave();
+
+                        // Set m.params
+                        ListBuffer<VarSymbol> params = new ListBuffer<VarSymbol>();
+                        JCVariableDecl lastParam = null;
+                        for (List<JCVariableDecl> l = tree.params; l.nonEmpty(); l = l.tail) {
+                            JCVariableDecl param = lastParam = l.head;
+                            assert param.sym != null;
+                            params.append(param.sym);
                         }
+                        tree.sym.params = params.toList();
+
+                        // mark the method varargs, if necessary
+                        if (lastParam != null && (lastParam.mods.flags & Flags.VARARGS) != 0)
+                            tree.sym.flags_field |= Flags.VARARGS;
+
+                        break;
                     }
-                    tree.sym.type = signature(tree.typarams, tree.params,
-                            tree.restype, tree.thrown,
-                            localEnv);
-                    tree.sym.flags_field &= ~FROMCLASS;
-                    localEnv.info.scope.leave();
-
-                    // Set m.params
-                    ListBuffer<VarSymbol> params = new ListBuffer<VarSymbol>();
-                    JCVariableDecl lastParam = null;
-                    for (List<JCVariableDecl> l = tree.params; l.nonEmpty(); l = l.tail) {
-                        JCVariableDecl param = lastParam = l.head;
-                        assert param.sym != null;
-                        params.append(param.sym);
-                    }
-                    tree.sym.params = params.toList();
-
-                    // mark the method varargs, if necessary
-                    if (lastParam != null && (lastParam.mods.flags & Flags.VARARGS) != 0)
-                        tree.sym.flags_field |= Flags.VARARGS;
-
-                    break;
                 }
             }
             if (tree.sym == m) {
@@ -678,8 +695,8 @@ public class MemberEnter extends JCTree.Visitor implements Completer {
             if (lastParam != null && (lastParam.mods.flags & Flags.VARARGS) != 0)
                 m.flags_field |= Flags.VARARGS;
 
-            chk.checkUnique(tree.pos(), m, enclScope);
-            enclScope.enter(m);
+            if( chk.checkUnique(tree.pos(), m, enclScope))
+                enclScope.enter(m);
         }
         annotateLater(tree.mods.annotations, localEnv, m);
         if (tree.defaultValue != null)
@@ -731,14 +748,17 @@ public class MemberEnter extends JCTree.Visitor implements Completer {
         tree.sym = v;
         if (tree.init != null) {
             v.flags_field |= HASINIT;
-            if ((v.flags_field & FINAL) != 0 && tree.init.getTag() != JCTree.NEWCLASS)
-                v.setLazyConstValue(initEnv(tree, env), log, attr, tree.init);
+            if ((v.flags_field & FINAL) != 0 && tree.init.getTag() != JCTree.NEWCLASS) {
+                Env<AttrContext> initEnv = getInitEnv(tree, env);
+                initEnv.info.enclVar = v;
+                v.setLazyConstValue(initEnv(tree, initEnv), log, attr, tree.init);
+            }
         }
         if (chk.checkUnique(tree.pos(), v, enclScope)) {
             chk.checkTransparentVar(tree.pos(), v, enclScope);
+            if (doEnterSymbol)
+                enclScope.enter(v);
         }
-        if (doEnterSymbol)
-            enclScope.enter(v);
         annotateLater(tree.mods.annotations, localEnv, v);
         v.pos = tree.pos;
     }
@@ -832,7 +852,7 @@ public class MemberEnter extends JCTree.Visitor implements Completer {
                             annotations.nonEmpty())
                             log.error(annotations.head.pos,
                                       "already.annotated",
-                                      Resolve.kindName(s), s);
+                                      kindName(s), s);
                         enterAnnotations(annotations, localEnv, s);
                     } finally {
                         log.useSource(prev);
@@ -929,6 +949,13 @@ public class MemberEnter extends JCTree.Visitor implements Completer {
         ClassSymbol c = (ClassSymbol)sym;
         ClassType ct = (ClassType)c.type;
         Env<AttrContext> env = enter.typeEnvs.get(c);
+
+        if (disabledCompletionInUnit == env.toplevel) {
+            // Re-install same completer for next time around and return.
+            sym.completer = this;
+            return;
+        }
+
         JCClassDecl tree = (JCClassDecl)env.tree;
         boolean wasFirst = isFirst;
         isFirst = false;
@@ -938,6 +965,9 @@ public class MemberEnter extends JCTree.Visitor implements Completer {
             // Save class environment for later member enter (2) processing.
             halfcompleted.append(env);
 
+            // Mark class as not yet attributed.
+            c.flags_field |= UNATTRIBUTED;
+
             // If this is a toplevel-class, make sure any preceding import
             // clauses have been seen.
             if (c.owner.kind == PCK) {
@@ -945,9 +975,6 @@ public class MemberEnter extends JCTree.Visitor implements Completer {
                 todo.append(env);
             }
 
-            // Mark class as not yet attributed.
-            c.flags_field |= UNATTRIBUTED;
-
             if (c.owner.kind == TYP)
                 c.owner.complete();
 
@@ -1026,7 +1053,7 @@ public class MemberEnter extends JCTree.Visitor implements Completer {
                 List<Type> thrown = List.nil();
                 long ctorFlags = 0;
                 boolean based = false;
-                if (c.name.len == 0) {
+                if (c.name.isEmpty()) {
                     JCNewClass nc = (JCNewClass)env.next.tree;
                     if (nc.constructor != null) {
                         Type superConstrType = types.memberType(c.type,
@@ -1170,7 +1197,7 @@ public class MemberEnter extends JCTree.Visitor implements Completer {
                             boolean based) {
         List<JCVariableDecl> params = make.Params(argtypes, syms.noSymbol);
         List<JCStatement> stats = List.nil();
-        if (c.type != syms.objectType)
+        if (!c.type.isErroneous() && c.type != syms.objectType)
             stats = stats.prepend(SuperCall(make, typarams, params, based));
         if ((c.flags() & ENUM) != 0 &&
             (types.supertype(c.type).tsym == syms.enumSym ||
@@ -1179,7 +1206,7 @@ public class MemberEnter extends JCTree.Visitor implements Completer {
             flags = (flags & ~AccessFlags) | PRIVATE | GENERATEDCONSTR;
         } else
             flags |= (c.flags() & AccessFlags) | GENERATEDCONSTR;
-        if (c.name.len == 0) flags |= ANONCONSTR;
+        if (c.name.isEmpty()) flags |= ANONCONSTR;
         JCTree result = make.MethodDef(
             make.Modifiers(flags),
             names.init,
