@@ -25,15 +25,20 @@
 package com.sun.tools.javac.comp;
 
 import com.sun.tools.javac.code.Flags;
+import com.sun.tools.javac.code.Kinds;
+import com.sun.tools.javac.code.Scope;
 import com.sun.tools.javac.code.Source;
 import com.sun.tools.javac.code.Symbol;
 import com.sun.tools.javac.code.Symbol.ClassSymbol;
+import com.sun.tools.javac.code.Symbol.MethodSymbol;
 import com.sun.tools.javac.code.Symtab;
 import com.sun.tools.javac.code.Type;
 import com.sun.tools.javac.code.Type.ClassType;
 import com.sun.tools.javac.code.TypeTags;
 import com.sun.tools.javac.code.Types;
 import com.sun.tools.javac.tree.JCTree;
+import com.sun.tools.javac.tree.JCTree.JCAssignOp;
+import com.sun.tools.javac.tree.JCTree.JCBinary;
 import com.sun.tools.javac.tree.JCTree.JCBlock;
 import com.sun.tools.javac.tree.JCTree.JCCase;
 import com.sun.tools.javac.tree.JCTree.JCClassDecl;
@@ -46,6 +51,7 @@ import com.sun.tools.javac.tree.JCTree.JCMethodInvocation;
 import com.sun.tools.javac.tree.JCTree.JCNewClass;
 import com.sun.tools.javac.tree.JCTree.JCStatement;
 import com.sun.tools.javac.tree.JCTree.JCTypeParameter;
+import com.sun.tools.javac.tree.JCTree.JCUnary;
 import com.sun.tools.javac.tree.JCTree.JCVariableDecl;
 import com.sun.tools.javac.tree.TreeInfo;
 import com.sun.tools.javac.tree.TreeMaker;
@@ -55,6 +61,7 @@ import com.sun.tools.javac.util.JCDiagnostic;
 import com.sun.tools.javac.util.JCDiagnostic.DiagnosticPosition;
 import com.sun.tools.javac.util.List;
 import com.sun.tools.javac.util.Log;
+import com.sun.tools.javac.util.MissingPlatformError;
 import com.sun.tools.javac.util.Name;
 import java.util.HashSet;
 import java.util.Set;
@@ -69,6 +76,7 @@ public class Repair extends TreeTranslator {
     /** The context key for the Repair phase. */
     protected static final Context.Key<Repair> repairKey = new Context.Key<Repair>();
     private static final String ERR_MESSAGE = "Uncompilable source code";
+    private static final Logger LOGGER = Logger.getLogger(Repair.class.getName());
 
     /** Get the instance for this context. */
     public static Repair instance(Context context) {
@@ -85,6 +93,7 @@ public class Repair extends TreeTranslator {
     private Types types;
     private Log log;
     private TreeMaker make;
+    private JCDiagnostic.Factory diags;
     private boolean allowEnums;
     
     private Env<AttrContext> attrEnv;
@@ -94,7 +103,8 @@ public class Repair extends TreeTranslator {
     private String classLevelErrMessage;
     private JCBlock staticInit;
     private List<JCTree> parents;
-    private Set<Name> repairedClassNames = new HashSet<Name>();
+    private Set<ClassSymbol> repairedClasses = new HashSet<ClassSymbol>();
+    private boolean isErrClass;
     
     private Repair(Context context) {
         context.put(repairKey, this);
@@ -103,6 +113,7 @@ public class Repair extends TreeTranslator {
         enter = Enter.instance(context);
         types = Types.instance(context);
         log = Log.instance(context);
+        diags = JCDiagnostic.Factory.instance(context);
         Source source = Source.instance(context);
         allowEnums = source.allowEnums();
     }
@@ -121,6 +132,11 @@ public class Repair extends TreeTranslator {
         } finally {
             parents = parents.tail;            
         }
+        if (tree.type != null && tree.type.isErroneous()) {
+            JCTree parent = parents.head;
+            if (parent == null || parent.getTag() != JCTree.CLASSDEF)
+                hasError = true;
+        }
         if (!(hasError && tree instanceof JCStatement))
             return tree;
         if (tree.getTag() == JCTree.CASE)
@@ -161,8 +177,6 @@ public class Repair extends TreeTranslator {
         }
     }
 
-
-
     @Override
     public void visitClassDef(JCClassDecl tree) {
         translateClass(tree.sym);
@@ -184,6 +198,7 @@ public class Repair extends TreeTranslator {
 
     @Override
     public void visitMethodDef(JCMethodDecl tree) {
+        boolean hadDuplicateSymError = hasError && err != null && "compiler.err.already.defined".equals(err.getCode()); //NOI18N
         tree.mods = translate(tree.mods);
         tree.restype = translate(tree.restype);
         tree.typarams = translateTypeParams(tree.typarams);
@@ -192,7 +207,17 @@ public class Repair extends TreeTranslator {
         tree.defaultValue = translate(tree.defaultValue);
         tree.body = translate(tree.body);
         result = tree;
-        if (hasError) {
+        if (isErrClass && tree.body != null) {
+            JCMethodInvocation app = TreeInfo.firstConstructorCall(tree);
+            Name meth = app != null ? TreeInfo.name(app.meth) : null;
+            Symbol sym = app != null ? TreeInfo.symbol(app.meth) : null;
+            if (meth != null && (meth == meth.table.names._this || meth == meth.table.names._super)
+                    && sym != null && sym.owner.getQualifiedName() != meth.table.names.java_lang_Enum)
+                tree.body.stats.tail = List.<JCStatement>nil();
+            else
+                tree.body.stats = List.<JCStatement>nil();
+        }
+        if (hasError && !hadDuplicateSymError) {
             if (tree.sym != null) {
                 tree.sym.flags_field &= ~(Flags.ABSTRACT | Flags.NATIVE);
                 tree.sym.defaultValue = null;
@@ -226,18 +251,10 @@ public class Repair extends TreeTranslator {
     public void visitApply(JCMethodInvocation tree) {
         Symbol meth = TreeInfo.symbol(tree.meth);
         if (meth == null) {
-            Logger.getLogger(Repair.class.getName()).warning("Repair.visitApply tree [" + tree + "] has null symbol."); //NOI18N
+            LOGGER.warning("Repair.visitApply tree [" + tree + "] has null symbol."); //NOI18N
             hasError = true;
         } else if (meth.type == null || meth.type.isErroneous()) {
             hasError = true;
-        } else {
-            List<Type> argtypes = meth.type.getParameterTypes();
-            if (allowEnums && meth.name == meth.name.table.init && meth.owner == syms.enumSym)
-                argtypes = argtypes.tail.tail;
-            if (tree.varargsElement == null && tree.args.length() != argtypes.length()) {
-                Logger.getLogger(Repair.class.getName()).warning("Repair.visitApply [" + meth.owner + "]'s method [" + meth + "] of type: [" + meth.type + "]; has different number of parameters than tree [" + tree +"]."); //NOI18N
-                hasError = true;
-            }
         }
         super.visitApply(tree);
     }
@@ -246,14 +263,44 @@ public class Repair extends TreeTranslator {
     public void visitNewClass(JCNewClass tree) {
         Symbol ctor = tree.constructor;
         if (ctor == null) {
-            Logger.getLogger(Repair.class.getName()).warning("Repair.visitNewClass tree [" + tree + "] has null constructor symbol."); //NOI18N
+            LOGGER.warning("Repair.visitNewClass tree [" + tree + "] has null constructor symbol."); //NOI18N
             hasError = true;
-        } else if (ctor.type == null || ctor.type.isErroneous()) {
+        } else if (tree.constructorType == null || tree.constructorType.isErroneous()) {
             hasError = true;
         }
         super.visitNewClass(tree);
     }
 
+    @Override
+    public void visitUnary(JCUnary tree) {
+        Symbol operator = tree.operator;
+        if (operator == null) {
+            LOGGER.warning("Repair.visitUnary tree [" + tree + "] has null operator symbol."); //NOI18N
+            hasError = true;
+        }
+        super.visitUnary(tree);
+    }
+
+    @Override
+    public void visitBinary(JCBinary tree) {
+        Symbol operator = tree.operator;
+        if (operator == null) {
+            LOGGER.warning("Repair.visitBinary tree [" + tree + "] has null operator symbol."); //NOI18N
+            hasError = true;
+        }
+        super.visitBinary(tree);
+    }
+
+    @Override
+    public void visitAssignop(JCAssignOp tree) {
+        Symbol operator = tree.operator;
+        if (operator == null) {
+            LOGGER.warning("Repair.visitAssignop tree [" + tree + "] has null operator symbol."); //NOI18N
+            hasError = true;
+        }
+        super.visitAssignop(tree);
+    }
+
     @Override
     public void visitCase(JCCase tree) {
         tree.pat = translate(tree.pat);
@@ -277,11 +324,22 @@ public class Repair extends TreeTranslator {
     private JCStatement generateErrStat(DiagnosticPosition pos, String msg) {
         make.at(pos);
         ClassType ctype = (ClassType)syms.runtimeExceptionType;
-        JCLiteral literal = make.Literal(msg != null ? ERR_MESSAGE + " - " + msg : ERR_MESSAGE); //NOI18N
-        JCNewClass tree = make.NewClass(null, null, make.QualIdent(ctype.tsym), List.<JCExpression>of(literal), null);
-        tree.constructor = rs.resolveConstructor(pos, attrEnv, ctype, List.of(literal.type), null, false, false);
-        tree.type = ctype;
-        return make.Throw(tree);
+        Symbol ctor = rs.resolveConstructor(pos, attrEnv, ctype, List.of(syms.stringType), null, false, false);
+        if (ctor.kind == Kinds.MTH) {
+            JCLiteral literal = make.Literal(msg != null ? ERR_MESSAGE + " - " + msg : ERR_MESSAGE); //NOI18N
+            JCNewClass tree = make.NewClass(null, null, make.QualIdent(ctype.tsym), List.<JCExpression>of(literal), null);
+            tree.type = ctype;
+            tree.constructor = ctor;
+            return make.Throw(tree);
+        }
+        ctor = rs.resolveConstructor(pos, attrEnv, ctype, List.<Type>nil(), null, false, false);
+        if (ctor.kind == Kinds.MTH) {
+            JCNewClass tree = make.NewClass(null, null, make.QualIdent(ctype.tsym), List.<JCExpression>nil(), null);
+            tree.type = ctype;
+            tree.constructor = ctor;
+            return make.Throw(tree);
+        }
+        throw new MissingPlatformError (diags.fragment("fatal.err.cant.locate.ctor", ctype)); //NOI18N
     }
     
     private JCExpression generateErrExpr(DiagnosticPosition pos, String msg) {
@@ -296,24 +354,36 @@ public class Repair extends TreeTranslator {
         return make.Block(Flags.STATIC, List.<JCStatement>of(generateErrStat(pos, msg)));
     }
 
+    private JCMethodDecl generateErrMethod(MethodSymbol sym) {
+        make.at(null);
+        return make.MethodDef(sym, make.Block(0, List.<JCStatement>of(generateErrStat(null, null))));
+    }
+
     private void translateClass(ClassSymbol c) {
         if (c == null)
             return;
         Type st = types.supertype(c.type);
         if (st.tag == TypeTags.CLASS)
             translateClass((ClassSymbol)st.tsym);
-        if (repairedClassNames.contains(c.flatname))
+        LOGGER.finest("Repair.translateClass: " + c); //NOI18N
+        if (repairedClasses.contains(c)) {
+            LOGGER.finest("Repair.translateClass: Should be already done"); //NOI18N
             return;
-        repairedClassNames.add(c.flatname);
+        }
         Env<AttrContext> myEnv = enter.typeEnvs.get(c);
-        if (myEnv == null)
+        if (myEnv == null) {
+            LOGGER.finest("Repair.translateClass: Context not found"); //NOI18N
             return;
+        }
+        LOGGER.finest("Repair.translateClass: Repairing " + c); //NOI18N
+        repairedClasses.add(c);
         Env<AttrContext> oldEnv = attrEnv;
         try {
             attrEnv = myEnv;
             TreeMaker oldMake = make;
             make = make.forToplevel(attrEnv.toplevel);
             boolean oldHasError = hasError;
+            boolean oldIsErrClass = isErrClass;
             JCDiagnostic oldErr = err;
             JCTree oldClassLevelErrTree = classLevelErrTree;
             String oldClassLevelErrMessage = classLevelErrMessage;
@@ -325,6 +395,7 @@ public class Repair extends TreeTranslator {
                         break;
                 }
                 hasError = false;
+                isErrClass = c.type.isErroneous();
                 err = null;
                 staticInit = null;
                 JCClassDecl tree = (JCClassDecl)attrEnv.tree;
@@ -332,18 +403,32 @@ public class Repair extends TreeTranslator {
                 tree.typarams = translateTypeParams(tree.typarams);
                 tree.extending = translate(tree.extending);
                 tree.implementing = translate(tree.implementing);
+                if (!hasError && (err = log.getErrDiag(tree)) != null) {
+                    hasError = true;
+                    isErrClass = true;
+                }
                 if (hasError && err != null) {
                     classLevelErrTree = err.getTree();
                     classLevelErrMessage = err.getMessage(null);
+                } else if (c.type.isErroneous() && oldHasError && oldErr != null) {
+                    classLevelErrTree = oldErr.getTree();
+                    classLevelErrMessage = oldErr.getMessage(null);
                 }
                 if (tree.defs != null) {
+                    HashSet<MethodSymbol> nonAbstractMethods = new HashSet<MethodSymbol>();
+                    for (Scope.Entry e = tree.sym.members_field.elems; e != null; e = e.sibling) {
+                        if (e.sym.kind == Kinds.MTH && (e.sym.flags_field & Flags.ABSTRACT) == 0 && e.sym.name != e.sym.name.table.names.clinit)
+                            nonAbstractMethods.add((MethodSymbol)e.sym);
+                    }
                     List<JCTree> last = null;
                     for (List<JCTree> l = tree.defs; l != null && l.nonEmpty(); l = l.tail) {
+                        if (l.head.getTag() == JCTree.METHODDEF)
+                            nonAbstractMethods.remove(((JCMethodDecl)l.head).sym);
                         hasError = false;
                         err = null;
                         l.head = translate(l.head);
                         if (hasError) {
-                            if (l.head.getTag() == JCTree.CLASSDEF) {
+                            if (l.head.getTag() == JCTree.CLASSDEF && tree.sym.members_field.includes(((JCClassDecl)l.head).sym)) {
                                 last = l;
                             } else {
                                 if (last != null)
@@ -369,12 +454,25 @@ public class Repair extends TreeTranslator {
                         else
                             tree.defs = tree.defs.prepend(generateErrStaticInit(classLevelErrTree, classLevelErrMessage));
                     }
+                    for (MethodSymbol symbol : nonAbstractMethods) {
+                        if ((symbol.owner.flags() & Flags.ENUM) != 0) {
+                            if ((symbol.name == symbol.name.table.names.values
+                                    && symbol.type.asMethodType().argtypes.isEmpty())
+                                    || (symbol.name == symbol.name.table.names.valueOf
+                                    && symbol.type.asMethodType().argtypes.head == enter.syms.stringType
+                                    && symbol.type.asMethodType().argtypes.tail.isEmpty())) {
+                                continue;
+                            }
+                        }
+                        tree.defs = tree.defs.prepend(generateErrMethod(symbol));
+                    }
                 }
             } finally {
                 staticInit = oldStaticinit;
                 classLevelErrTree = oldClassLevelErrTree;
                 classLevelErrMessage = oldClassLevelErrMessage;
                 err = oldErr;
+                isErrClass = oldIsErrClass;
                 hasError = oldHasError;
                 make = oldMake;
             }
@@ -397,6 +495,6 @@ public class Repair extends TreeTranslator {
     }
     
     public void flush() {
-        repairedClassNames.clear();
+        repairedClasses.clear();
     }
 }
