@@ -1,5 +1,5 @@
 /*
- * Copyright 1999-2006 Sun Microsystems, Inc.  All Rights Reserved.
+ * Copyright 1999-2008 Sun Microsystems, Inc.  All Rights Reserved.
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS FILE HEADER.
  *
  * This code is free software; you can redistribute it and/or modify it
@@ -44,6 +44,7 @@ import com.sun.tools.javac.code.*;
 import com.sun.tools.javac.code.Type.*;
 import com.sun.tools.javac.code.Symbol.*;
 import com.sun.tools.javac.code.Symtab;
+import com.sun.tools.javac.file.BaseFileObject;
 import com.sun.tools.javac.util.*;
 import com.sun.tools.javac.util.List;
 
@@ -70,7 +71,7 @@ public class ClassReader extends ClassFile implements Completer {
     protected static final Context.Key<ClassReader> classReaderKey =
         new Context.Key<ClassReader>();
 
-    protected Annotate annotate;
+    Annotate annotate;
 
     /** Switch: verbose output.
      */
@@ -88,19 +89,15 @@ public class ClassReader extends ClassFile implements Completer {
 
     /** Switch: read GJ signature information.
      */
-    protected boolean allowGenerics;
-
-    /** Switch: Allow GJ covariant return types.
-     */
-    protected boolean allowCovRetTypes;
+    boolean allowGenerics;
 
     /** Switch: read varargs attribute.
      */
-    protected boolean allowVarargs;
+    boolean allowVarargs;
 
     /** Switch: allow annotations.
      */
-    protected boolean allowAnnotations;
+    boolean allowAnnotations;
 
     /** Switch: preserve parameter names from the variable table.
      */
@@ -127,7 +124,7 @@ public class ClassReader extends ClassFile implements Completer {
     Types types;
 
     /** The name table. */
-    final Name.Table names;
+    final Names names;
 
     /** Force a completion failure on this name
      */
@@ -136,7 +133,13 @@ public class ClassReader extends ClassFile implements Completer {
     /** Access to files
      */
     private final JavaFileManager fileManager;
+
+    /** Factory for diagnostics
+     */
+    JCDiagnostic.Factory diagFactory;
+
     private final ClassNamesForFileOraculum classNamesOraculum;
+    private final boolean ideMode;
 
     /** Can be reassigned from outside:
      *  the completer to be used for ".java" files. If this remains unassigned
@@ -222,13 +225,13 @@ public class ClassReader extends ClassFile implements Completer {
     protected ClassReader(Context context, boolean definitive) {
         if (definitive) context.put(classReaderKey, this);
 
-        names = Name.Table.instance(context);
+        names = Names.instance(context);
         syms = Symtab.instance(context);
         types = Types.instance(context);
         fileManager = context.get(JavaFileManager.class);
         if (fileManager == null)
             throw new AssertionError("FileManager initialization error");
-
+        diagFactory = JCDiagnostic.Factory.instance(context);
         classNamesOraculum = context.get(ClassNamesForFileOraculum.class);
         
         init(syms, definitive);
@@ -238,11 +241,11 @@ public class ClassReader extends ClassFile implements Completer {
         annotate = Annotate.instance(context);
         verbose        = options.get("-verbose")        != null;
         checkClassFile = options.get("-checkclassfile") != null;
-        boolean ideMode = options.get("ide") != null;
+        ideMode = options.get("ide") != null;
         Source source = Source.instance(context);
-        allowGenerics    = allowCovRetTypes = source.allowGenerics() || ideMode;
-        allowVarargs     = source.allowVarargs() || ideMode;
-        allowAnnotations = source.allowAnnotations() || ideMode;
+        allowGenerics    = source.allowGenerics();
+        allowVarargs     = source.allowVarargs();
+        allowAnnotations = source.allowAnnotations();
         saveParameterNames = options.get("save-parameter-names") != null;
         cacheCompletionFailure = options.get("dev") == null;
         preferSource = "source".equals(options.get("-Xprefer"));
@@ -257,7 +260,7 @@ public class ClassReader extends ClassFile implements Completer {
 
     /** Add member to class unless it is synthetic.
      */
-    protected void enterMember(ClassSymbol c, Symbol sym) {
+    private void enterMember(ClassSymbol c, Symbol sym) {
         if ((sym.flags_field & (SYNTHETIC|BRIDGE)) != SYNTHETIC)
             c.members_field.enter(sym);
     }
@@ -266,23 +269,26 @@ public class ClassReader extends ClassFile implements Completer {
  * Error Diagnoses
  ***********************************************************************/
 
-    public static class BadClassFile extends CompletionFailure {
+
+    public class BadClassFile extends CompletionFailure {
         private static final long serialVersionUID = 0;
 
-        /**
-         * @param msg A localized message.
-         */
-        public BadClassFile(ClassSymbol c, Object cname, Object msg) {
-            super(c, Log.getLocalizedString("bad.class.file.header",
-                                            cname, msg));
+        public BadClassFile(TypeSymbol sym, JavaFileObject file, JCDiagnostic diag) {
+            super(sym, createBadClassFileDiagnostic(file, diag));
         }
     }
+    // where
+    private JCDiagnostic createBadClassFileDiagnostic(JavaFileObject file, JCDiagnostic diag) {
+        String key = (file.getKind() == JavaFileObject.Kind.SOURCE
+                    ? "bad.source.file.header" : "bad.class.file.header");
+        return diagFactory.fragment(key, file, diag);
+    }
 
     public BadClassFile badClassFile(String key, Object... args) {
         return new BadClassFile (
             currentOwner.enclClass(),
             currentClassFile,
-            Log.getLocalizedString(key, args));
+            diagFactory.fragment(key, args));
     }
 
 /************************************************************************
@@ -517,14 +523,6 @@ public class ClassReader extends ClassFile implements Completer {
     int siglimit;
     boolean sigEnterPhase = false;
 
-    /** Convert signature to type, where signature is a name.
-     */
-    Type sigToType(Name sig) {
-        return sig == null
-            ? null
-            : sigToType(sig.table.names, sig.index, sig.len);
-    }
-
     /** Convert signature to type, where signature is a byte array segment.
      */
     Type sigToType(byte[] sig, int offset, int len) {
@@ -653,7 +651,7 @@ public class ClassReader extends ClassFile implements Completer {
                                                          startSbp,
                                                          sbp - startSbp));
                 if (err)
-                    outer = new ErrorType(t, false);
+                    outer = new ErrorType(Type.noType, t, false);
                 else if (outer == Type.noType)
                     outer = t.erasure(types);
                 else
@@ -746,12 +744,6 @@ public class ClassReader extends ClassFile implements Completer {
         return head.tail;
     }
 
-    /** Convert signature to type parameters, where signature is a name.
-     */
-    List<Type> sigToTypeParams(Name name) {
-        return sigToTypeParams(name.table.names, name.index, name.len);
-    }
-
     /** Convert signature to type parameters, where signature is a byte
      *  array segment.
      */
@@ -875,17 +867,17 @@ public class ClassReader extends ClassFile implements Completer {
                 sym.flags_field |= SYNTHETIC;
         } else if (attrName == names.Bridge) {
             sym.flags_field |= BRIDGE;
-            if (!allowCovRetTypes)
+            if (!allowGenerics)
                 sym.flags_field &= ~SYNTHETIC;
         } else if (attrName == names.Deprecated) {
             sym.flags_field |= DEPRECATED;
         } else if (attrName == names.Varargs) {
-            if (allowVarargs) sym.flags_field |= VARARGS;
+            if (allowVarargs || ideMode) sym.flags_field |= VARARGS;
         } else if (attrName == names.Annotation) {
-            if (allowAnnotations) sym.flags_field |= ANNOTATION;
+            if (allowAnnotations || ideMode) sym.flags_field |= ANNOTATION;
         } else if (attrName == names.Enum) {
             sym.flags_field |= ENUM;
-        } else if (allowGenerics && attrName == names.Signature) {
+        } else if ((allowGenerics || ideMode) && attrName == names.Signature) {
             List<Type> thrown = sym.type.getThrownTypes();
             sym.type = readType(nextChar());
             //- System.err.println(" # " + sym.type);
@@ -907,6 +899,7 @@ public class ClassReader extends ClassFile implements Completer {
                 int firstParam = ((sym.flags() & STATIC) == 0) ? 1 : 0;
                 int endParam = firstParam + Code.width(sym.type.getParameterTypes());
                 int numEntries = nextChar();
+                int lastNameIndex = -1;
                 for (int i=0; i<numEntries; i++) {
                     int start_pc = nextChar();
                     int length = nextChar();
@@ -918,9 +911,14 @@ public class ClassReader extends ClassFile implements Completer {
                         register < endParam) {
                         int index = firstParam;
                         for (Type t : sym.type.getParameterTypes()) {
-                            if (index == register) {
-                                parameterNames = parameterNames.prepend(readName(nameIndex));
-                                break;
+                            if (index > lastNameIndex) {
+                                lastNameIndex = index;
+                                if (index == register) {
+                                    parameterNames = parameterNames.prepend(readName(nameIndex));
+                                    break;
+                                } else {
+                                    parameterNames = parameterNames.prepend(null);
+                                }
                             }
                             index += Code.width(t);
                         }
@@ -975,7 +973,7 @@ public class ClassReader extends ClassFile implements Completer {
 
         self.name = simpleBinaryName(self.flatname, c.flatname) ;
         self.owner = m != null ? m : c;
-        if (self.name.len == 0)
+        if (self.name.isEmpty())
             self.fullname = self.name;
         else
             self.fullname = ClassSymbol.formFullName(self.name, self.owner);
@@ -1001,7 +999,11 @@ public class ClassReader extends ClassFile implements Completer {
 
     // See java.lang.Class
     protected Name simpleBinaryName(Name self, Name enclosing) {
-        String simpleBinaryName = self.toString().substring(enclosing.toString().length());
+        String selfStr = self.toString();
+        String enclStr = enclosing.toString();
+        if (selfStr.length() <= enclStr.length())
+            throw badClassFile("bad.enclosing.method", self);
+        String simpleBinaryName = selfStr.substring(enclStr.length());
         if (simpleBinaryName.length() < 1 || simpleBinaryName.charAt(0) != '$')
             throw badClassFile("bad.enclosing.method", self);
         int index = 1;
@@ -1079,10 +1081,10 @@ public class ClassReader extends ClassFile implements Completer {
     void readClassAttr(ClassSymbol c, Name attrName, int attrLen) {
         if (attrName == names.SourceFile) {
             Name n = readName(nextChar());
-            c.sourcefile = new SourceFileObject(n);
+            c.sourcefile = new SourceFileObject(n, c.flatname);
         } else if (attrName == names.InnerClasses) {
             readInnerClasses(c);
-        } else if (allowGenerics && attrName == names.Signature) {
+        } else if ((allowGenerics || ideMode) && attrName == names.Signature) {
             readingClassAttr = true;
             try {
                 ClassType ct1 = (ClassType)c.type;
@@ -1354,7 +1356,7 @@ public class ClassReader extends ClassFile implements Completer {
                     log.warning("annotation.method.not.found.reason",
                                 container,
                                 name,
-                                failure.getMessage());
+                                failure.getDetailValue());//diagnostic, if present
                 }
             } finally {
                 log.useSource(prevSource);
@@ -1523,7 +1525,7 @@ public class ClassReader extends ClassFile implements Completer {
             // Sometimes anonymous classes don't have an outer
             // instance, however, there is no reliable way to tell so
             // we never strip this$n
-            if (currentOwner.name.len != 0)
+            if (!currentOwner.name.isEmpty())
                 type = new MethodType(type.getParameterTypes().tail,
                                       type.getReturnType(),
                                       type.getThrownTypes(),
@@ -1715,7 +1717,7 @@ public class ClassReader extends ClassFile implements Completer {
         if ((flags & ACC_BRIDGE) != 0) {
             flags &= ~ACC_BRIDGE;
             flags |= BRIDGE;
-            if (!allowCovRetTypes)
+            if (!allowGenerics)
                 flags &= ~SYNTHETIC;
         }
         if ((flags & ACC_VARARGS) != 0) {
@@ -1893,9 +1895,16 @@ public class ClassReader extends ClassFile implements Completer {
         if (classfile != null) {
             JavaFileObject previousClassFile = currentClassFile;
             try {
-                assert !filling :
-                    "Filling " + classfile.toUri() +
-                    " during " + previousClassFile;
+                if (filling) {
+                    StringBuilder sb = new StringBuilder();
+                    for (StackTraceElement[] trace : Thread.getAllStackTraces().values()) {
+                        for (StackTraceElement element : trace)
+                            sb.append(element).append('\n');
+                        sb.append('\n');
+                    }
+                    assert false : "Filling " + classfile.toUri() +
+                    " during " + previousClassFile + "\n" + sb;
+                }
                 currentClassFile = classfile;
                 if (verbose) {
                     printVerbose("loading", currentClassFile.toString());
@@ -1930,7 +1939,9 @@ public class ClassReader extends ClassFile implements Completer {
                     }
                 } else {
                     if (sourceCompleter != null) {
-                        sourceCompleter.complete(c);
+                        if (!classfile.isNameCompatible("package-info", JavaFileObject.Kind.SOURCE)) {
+                            sourceCompleter.complete(c);
+                        }
                     } else {
                         throw new IllegalStateException("Source completer required to read "
                                                         + classfile.toUri());
@@ -1943,10 +1954,10 @@ public class ClassReader extends ClassFile implements Completer {
                 currentClassFile = previousClassFile;
             }
         } else {
+            JCDiagnostic diag =
+                diagFactory.fragment("class.file.not.found", c.flatname);
             throw
-                newCompletionFailure(c,
-                                     Log.getLocalizedString("class.file.not.found",
-                                                            c.flatname));
+                newCompletionFailure(c, diag);
         }
     }
     // where
@@ -1984,22 +1995,22 @@ public class ClassReader extends ClassFile implements Completer {
          *  In practice, only one can be used at a time, so we share one
          *  to reduce the expense of allocating new exception objects.
          */
-        private CompletionFailure newCompletionFailure(ClassSymbol c,
-                                                       String localized) {
+        private CompletionFailure newCompletionFailure(TypeSymbol c,
+                                                       JCDiagnostic diag) {
             if (!cacheCompletionFailure) {
                 // log.warning("proc.messager",
                 //             Log.getLocalizedString("class.file.not.found", c.flatname));
                 // c.debug.printStackTrace();
-                return new CompletionFailure(c, localized);
+                return new CompletionFailure(c, diag);
             } else {
                 CompletionFailure result = cachedCompletionFailure;
                 result.sym = c;
-                result.errmsg = localized;
+                result.diag = diag;
                 return result;
             }
         }
         private CompletionFailure cachedCompletionFailure =
-            new CompletionFailure(null, null);
+            new CompletionFailure(null, (JCDiagnostic) null);
         {
             cachedCompletionFailure.setStackTrace(new StackTraceElement[0]);
         }
@@ -2101,14 +2112,10 @@ public class ClassReader extends ClassFile implements Completer {
         c.flags_field |= seen;
     }
 
-    private boolean isSigOverClass (final JavaFileObject a, final JavaFileObject b) {
+    private boolean isSigOverClass(final JavaFileObject a, final JavaFileObject b) {
         String patha = a.getName().toLowerCase();
         String pathb = b.getName().toLowerCase();
-        if (pathb.endsWith(".sig"))  //NOI18N
-            return patha.endsWith(".class");  //NOI18N
-        else if (pathb.endsWith(".class"))  //NOI18N
-            return patha.endsWith(".class") && b.getLastModified() >= a.getLastModified();  //NOI18N
-        return false;
+        return pathb.endsWith(".sig") && patha.endsWith(".class");  //NOI18N
     }
 
     /** Implement policy to choose to derive information from a source
@@ -2293,9 +2300,12 @@ public class ClassReader extends ClassFile implements Completer {
         /** The file's name.
          */
         private Name name;
+        private Name flatname;
 
-        public SourceFileObject(Name name) {
+        public SourceFileObject(Name name, Name flatname) {
+            super(null); // no file manager; never referenced for this file object
             this.name = name;
+            this.flatname = flatname;
         }
 
         public InputStream openInputStream() {
@@ -2357,5 +2367,9 @@ public class ClassReader extends ClassFile implements Completer {
             throw new UnsupportedOperationException();
         }
 
+        @Override
+        protected String inferBinaryName(Iterable<? extends File> path) {
+            return flatname.toString();
+        }
     }
 }
